@@ -21,11 +21,10 @@ BEGIN {
 use Test::Unit::Lite;
 
 use Exception::Base
-    max_arg_nums => 0, max_arg_len => 200, verbosity => 3,
-    '+ignore_package' => [ qr/^Test::Unit::/, 'main' ],
-    'Exception::Base::Warning';
+    max_arg_nums => 0, max_arg_len => 200, verbosity => 4,
+    'Exception::DiedTest::Warning';
 use Exception::Died '%SIG';
 
-local $SIG{__WARN__} = sub { Exception::Base::Warning->throw($_[0], ignore_level => 1) };
+local $SIG{__WARN__} = sub { Exception::DiedTest::Warning->throw($_[0], ignore_level => 1) };
 
 all_tests;
